package WWW::Mechanize::Plugin::DOM;

# DOM is in a separate module from JavaScript because other scripting
# languages may use DOM as well. Anyone have time to implement Acme::Chef
# bindings for Mech? :-)

$VERSION = '0.000002';

use strict;
use warnings;

use Scalar::Util 'weaken';

require HTML::DOM;
require URI;


sub init { # expected to return a plugin object that the mech object will
           # use to communicate with the plugin.

	my ($package, $mech, @opts) = @_;

	my $self = bless {
		script_handlers => {},
		event_attr_handlers => {},
	}, $package;

	$mech->add_handler(
		parse_html => \&_parse_html
	);
	$mech->add_handler( get_content =>
		sub { $$self{tree}->documentElement->as_HTML }
	);
	$mech->add_handler( get_text_content =>
		sub { $$self{tree}->documentElement->as_text }
	);

#	$self->options(@opts);
#warn $self;
	$self;
}

sub _parse_html {
	my ($mech,$src) = @_;
	weaken $mech;
	my $self = $mech->plugin('DOM');

	$$self{tree} = my $tree = new HTML::DOM;

	if(%{$$self{script_handlers}} || %{$$self{event_attr_handlers}}) {
		my $script_type = $mech->response->header(
			'Content-Script-Type');
		defined $script_type or $tree->elem_handler(meta =>
		    sub {
			my($tree, $elem) = @_;
			return unless lc $elem->attr('http-equiv')
				eq 'content-script-type';
			$script_type = $elem->attr('content');
		});

		if(%{$$self{script_handlers}}) {
			$tree->elem_handler(script => sub {
			    my($tree, $elem) = @_;

			    my $lang = $elem->attr('type');
			    defined $lang
			        or $lang = $elem->attr('language');
			    defined $lang or $lang = $script_type;

			    my $uri;
			    my($inline, $code, ) = 0;
			    if($uri = $elem->attr('src')) {
return; # ~~~ This is currently causing infinite recursion. I have to get
        #     to the bottom of this, and then re-enable it.
			        # ~~~ Is there some way to get the
			        #     Mech object to do this with-
			        #    out pushing the page stack?
			        require LWP::Simple;
			        require URI;
			        my $base = $mech->base;
   			        $uri = URI->new_abs( $uri, $base )
			            if $base;
			        defined(
			           $code = LWP::Simple::get($uri)
			        ) or $mech->warn("couldn't get script $uri"),return;
			        # ~~~ I probably need to provide better
			        #     diagnostics. Maybe I can't use
			        #     LWP::Simple.
			    }
			    else {
			        $code = $elem->firstChild->data;
			        ++$inline;
			        $uri = $mech->uri;
			    };
	
			    SCRIPT_HANDLER: {
			    while(my($lang_re,$handler) = each
			          %{$$self{script_handlers}}) {
			        next if $lang_re eq 'default';
			        $lang =~ $lang_re and
			            &$handler($tree, $code,
					$uri, 1, $inline),
			        # ~~~ That line number (1) is currently
			        #    invalid for almost all inline scripts.
			            last SCRIPT_HANDLER;
			    } # end of while
			    &{ $$self{script_handlers}{default} ||
			        return }($tree, $code,
					$uri, 1, $inline);
			        # ~~~ That line number (1) is currently
			        #    invalid for almost all inline scripts.
			    } # end of S_H
			});

			$tree->elem_handler(noscript => sub {
				$_[1]->detach#->delete;
				# ~~~ delete currently stops it from work-
				#     ing; I need to looook into this.
			});
		}

		if(%{$$self{event_attr_handlers}}) {
			$tree->event_attr_handler(sub {
				my($elem, $event, $code) = @_;
				my $lang = $elem->attr('language');
				defined $lang or $lang = $script_type;
	
				HANDLER: {
				while(my($lang_re,$handler) = each
				    %{$$self{event_attr_handlers}}) {
					next if $lang_re eq 'default';
					$lang =~ $lang_re and
					    &$handler($elem, $event,$code),
					    last HANDLER;
				} # end of while
				&{ $$self{event_attr_handlers}{default} ||
					return }($elem, $event,$code);
				} # end of HANDLER
			});
		}
	}
	# ~~~ Should we use the content of <noscript> elems if no script
	#     handler is provided but an event attribute handler *is*
	#     provided? (Now who would be crazy enough to do that?)
	if(!%{$$self{script_handlers}}) {
		$tree->elem_handler(noscript => sub {
			$_[1]->replace_with_content->delete;
			# ~~~ why does this need delete?
		});
	}

	$tree->parse($src);
	$tree->eof;

	return 1;
}

sub options {
	my($self,%opts) = @_;
	for (keys %opts) {
		if($_ eq 'script_handlers') {
			%{$$self{script_handlers}} = (
				%{$$self{script_handlers}}, %{$opts{$_}}
			);
		}
		elsif($_ eq 'event_attr_handlers') {
			%{$$self{event_attr_handlers}} = (
			    %{$$self{event_attr_handlers}},
			    %{$opts{$_}}
			);
		}
	}
}

sub tree { $_[0]{tree} }


=head1 NAME

WWW::Mechanize::Plugin::DOM - HTML Document Object Model plugin for Mech

=head1 SYNOPSIS

  use WWW::Mechanize;

  my $m = new WWW::Mechanize;

  $m->use_plugin('DOM',
      script_handlers => {
          default => \&script_handler,
          qr/(?:^|\/)(?:x-)?javascript/ => \&script_handler,
      },
      event_attr_handlers => {
          default => \&event_attr_handler,
          qr/(?:^|\/)(?:x-)?javascript/ => \&event_attr_handler,
      },
  );

  sub script_handler {
          my($dom_tree, $code, $url, $line, $is_inline) = @_;
          # ... code to run the script ...
  }

  sub event_attr_handler {
          my($elem, $event_name, $code) = @_;
          # ... code that returns a coderef ...
  }

  $m->plugin('DOM')->tree; # DOM tree for the current page

=head1 DESCRIPTION

blah blah blah

(event_attr|script)_handlers => {default => ... } is used when the script 
elem has no
'type' or 'language' attribute, and there is no Content-Script-Type header.

=head1 BUGS

The line number passed to script handlers is currently always 1,
which is usually wrong 
if the
script is inline.
